import model.Either;
import model.Leaf;
import model.Node;
import model.Tree;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by user on 29.04.2017.
 */
public class TreeWorkerImpl implements TreeWorker {
    @Override
    public int depth(Tree<?> tree) {
        if (tree == null)
            return 0;
        while (tree.toEither().isRight()) {
            Either<Leaf, Node> leafNodeEither = tree.toEither().right();
            while (leafNodeEither.isLeft()) {
                return 1;
            }
            while (leafNodeEither.isRight()) {
                Node node = leafNodeEither.right();
                return 1 + Math.max(depth(node.left), depth(node.right));
            }
        }
        return 0;
    }

    @Override
    public <T> boolean contains(Tree<T> tree, T value) {
        if (tree == null)
            return false;
        while (tree.toEither().isRight()) {
            Either<Leaf, Node> leafNodeEither = tree.toEither().right();
            while (leafNodeEither.isLeft()) {
                Leaf leaf = leafNodeEither.left();
                return value.equals(leaf.value);
            }
            while (leafNodeEither.isRight()) {
                Node node = leafNodeEither.right();
                return contains(node.left, value) | contains(node.right, value);
            }
        }
        return false;
    }

    @Override
    public <T> Tree<T> node(Tree<T> tree, T value) {
        if (tree == null)
            return null;
        while (tree.toEither().isRight()) {
            Either<Leaf, Node> leafNodeEither = tree.toEither().right();
            while (leafNodeEither.isRight()) {
                Node tmpNode = leafNodeEither.right();
                if (contains(tmpNode.left, value) | contains(tmpNode.right, value)) {
                    return tmpNode;
                } else {
                    return null;
                }
            }
        }
        return null;
    }

    @Override
    public <T> List<T> path(Tree<T> tree, T value) {
        List<T> tList = new ArrayList<>();
        if (tree == null)
            return null;
        while (tree.toEither().isRight()) {
            Either<Leaf, Node> leafNodeEither = tree.toEither().right();
            while (leafNodeEither.isRight()) {
                Node tmpNode = leafNodeEither.right();
                if (contains(tmpNode.left, value) | contains(tmpNode.right, value)) {
                    tList.add((T) tmpNode.value);
                } else if (leafNodeEither.isRight()) {
                    tmpNode = leafNodeEither.right();
                } else {
                    return tList;
                }
            }
        }
        return tList;
    }
}
